@@ -26,7 +26,7 @@ import math
 import sys
 
 from settings import SSH_FORWARDER, SSH_FORWARDER_PORT, CONFIG_ADDON
-from datetime import datetime
+from datetime import datetime, timedelta
 
 from pycclib.cclib import GoneError, ForbiddenError, TokenRequiredError, \
     BadRequestError, ConflictDuplicateError, UnauthorizedError, \
@@ -185,7 +185,6 @@ class AppController():
             Creates a new application.
         """
         try:
-            #noinspection PyTupleAssignmentBalance
             app_name, deployment_name = self.parse_app_deployment_name(args.name)
         except ParseAppDeploymentName:
             raise InputErrorException('InvalidApplicationName')
@@ -244,15 +243,14 @@ class AppController():
             to delete the app which is a rather bad user experience.
         """
         if self.api.check_token():
-            #noinspection PyTupleAssignmentBalance
             app_name, deployment_name = self.parse_app_deployment_name(args.name)
             if not self.does_app_exist(app_name):
                 raise InputErrorException('WrongApplication')
             if deployment_name:
                 raise InputErrorException('DeleteOnlyApplication')
             if not args.force_delete:
-                question = raw_input('Do you really want to delete this ' +
-                                     'application? Type "Yes" without the quotes to delete: ')
+                question = raw_input("Do you really want to delete application '{0}'? ".format(app_name) +
+                                     'Type "Yes" without the quotes to delete: ')
             else:
                 question = 'Yes'
             if question.lower() == 'yes':
@@ -336,7 +334,7 @@ class AppController():
     def _get_url(self, deployment):
         return "http://{0}".format(deployment['default_subdomain'])
 
-    def _open(self, app_or_deployment_name):
+    def get_deployment_from_app_deployment_name(self, app_or_deployment_name):
         app_name, deployment_name = self.parse_app_deployment_name(app_or_deployment_name)
         if not deployment_name:
             deployment_name = 'default'
@@ -351,6 +349,15 @@ class AppController():
         else:
             return app_name, deployment_name, deployment
 
+    def _open(self, url):
+        savout = os.dup(1)
+        os.close(1)
+        os.open(os.devnull, os.O_RDWR)
+        try:
+            webbrowser.open_new_tab(url)
+        finally:
+            os.dup2(savout, 1)
+
     def open(self, args):
         """
             Open deployment URL on the default browser.
@@ -361,15 +368,9 @@ class AppController():
 
             'cctrlapp APP_NAME/DEP_NAME open' opens the deployment's URL
         """
-        app_name, deployment_name, obj = self._open(args.name)
+        app_name, deployment_name, obj = self.get_deployment_from_app_deployment_name(args.name)
         url = self._get_url(obj)
-        savout = os.dup(1)
-        os.close(1)
-        os.open(os.devnull, os.O_RDWR)
-        try:
-            webbrowser.open_new_tab(url)
-        finally:
-            os.dup2(savout, 1)
+        self._open(url)
 
     def _get_size_from_memory(self, memory):
         res = re.match(r'(\d+)(.*)', memory.lower())
@@ -395,7 +396,6 @@ class AppController():
             other arguments were passed at the command line.
         """
         try:
-            #noinspection PyTupleAssignmentBalance
             app_name, deployment_name = self.parse_app_deployment_name(args.name)
         except ParseAppDeploymentName:
             raise InputErrorException('InvalidApplicationName')
@@ -445,20 +445,34 @@ class AppController():
         else:
             return True
 
+    def redeploy(self, deployment):
+        """
+            Redeploy an existing application with same options and configuration.
+        """
+        try:
+            app_name, deployment_name = self.parse_app_deployment_name(deployment['name'])
+            self.api.update_deployment(
+                app_name,
+                version=-1,
+                deployment_name=deployment_name,
+                min_boxes=deployment['min_boxes'],
+                max_boxes=deployment['max_boxes'],
+                stack=deployment['stack']['name'])
+        except (KeyError, GoneError, BadRequestError):
+            raise InputErrorException('DeploymentFailed')
+
     def undeploy(self, args):
         """
             Undeploys the deployment, deletes the database and files.
         """
-        #noinspection PyTupleAssignmentBalance
         app_name, deployment_name = self.parse_app_deployment_name(args.name)
         if not deployment_name:
             raise InputErrorException('NoDeployment')
         if not self.does_app_exist(app_name):
             raise InputErrorException('WrongApplication')
         if not args.force_delete:
-                question = raw_input('Do you really want to delete this ' +
-                                     'deployment? This will delete everything including files ' +
-                                     'and the database. Type "Yes" without the quotes to delete: ')
+            question = raw_input("Do you really want to delete deployment '{0}'? ".format(args.name) +
+                                 'This will delete everything including files and the database. Type "Yes" without the quotes to delete: ')
         else:
             question = 'Yes'
         if question.lower() == 'yes':
@@ -475,7 +489,6 @@ class AppController():
         """
             Adds the given alias to the deployment.
         """
-        #noinspection PyTupleAssignmentBalance
         app_name, deployment_name = self.parse_app_deployment_name(args.name)
         if not deployment_name:
             raise InputErrorException('NoDeployment')
@@ -488,7 +501,6 @@ class AppController():
         """
             Shows the details of an alias.
         """
-        #noinspection PyTupleAssignmentBalance
         app_name, deployment_name = self.parse_app_deployment_name(args.name)
         if not deployment_name:
             raise InputErrorException('NoDeployment')
@@ -512,7 +524,6 @@ class AppController():
         """
             Removes an alias form a deployment.
         """
-        #noinspection PyTupleAssignmentBalance
         app_name, deployment_name = self.parse_app_deployment_name(args.name)
         if not deployment_name:
             raise InputErrorException('NoDeployment')
@@ -528,7 +539,6 @@ class AppController():
         """
             Adds the given worker to the deployment.
         """
-        #noinspection PyTupleAssignmentBalance
         app_name, deployment_name = self.parse_app_deployment_name(args.name)
         if args.size:
             size = args.size
@@ -564,7 +574,6 @@ class AppController():
         """
             Shows the details of an worker.
         """
-        #noinspection PyTupleAssignmentBalance
         app_name, deployment_name = self.parse_app_deployment_name(args.name)
         if not deployment_name:
             raise InputErrorException('NoDeployment')
@@ -588,7 +597,6 @@ class AppController():
         """
             Removes an worker form a deployment.
         """
-        #noinspection PyTupleAssignmentBalance
         app_name, deployment_name = self.parse_app_deployment_name(args.name)
         if not deployment_name:
             raise InputErrorException('NoDeployment')
@@ -602,7 +610,6 @@ class AppController():
         """
             Adds the given worker to the deployment.
         """
-        #noinspection PyTupleAssignmentBalance
         app_name, deployment_name = self.parse_app_deployment_name(args.name)
         if not deployment_name:
             raise InputErrorException('NoDeployment')
@@ -618,7 +625,6 @@ class AppController():
         """
             Shows the details of an worker.
         """
-        #noinspection PyTupleAssignmentBalance
         app_name, deployment_name = self.parse_app_deployment_name(args.name)
         if not deployment_name:
             raise InputErrorException('NoDeployment')
@@ -642,7 +648,6 @@ class AppController():
         """
             Removes an worker form a deployment.
         """
-        #noinspection PyTupleAssignmentBalance
         app_name, deployment_name = self.parse_app_deployment_name(args.name)
         if not deployment_name:
             raise InputErrorException('NoDeployment')
@@ -663,7 +668,6 @@ class AppController():
         """
             Shows the config variables
         """
-        #noinspection PyTupleAssignmentBalance
         app_name, deployment_name = self.parse_app_deployment_name(args.name)
         if not deployment_name:
             raise InputErrorException('NoDeployment')
@@ -681,7 +685,6 @@ class AppController():
         """
             Adds the given variable and value to the config addon.
         """
-        #noinspection PyTupleAssignmentBalance
         app_name, deployment_name = self.parse_app_deployment_name(args.name)
         if not deployment_name:
             raise InputErrorException('NoDeployment')
@@ -721,7 +724,6 @@ class AppController():
         """
             Remove the given variable if it exists.
         """
-        #noinspection PyTupleAssignmentBalance
         app_name, deployment_name = self.parse_app_deployment_name(args.name)
         if not deployment_name:
             raise InputErrorException('NoDeployment')
@@ -742,7 +744,6 @@ class AppController():
 
         return True
 
-    #noinspection PyUnusedLocal
     def listAddons(self, args):
         """
             Returns a list of all available addons
@@ -755,26 +756,33 @@ class AppController():
         """
             Adds the given addon to the deployment.
         """
-        #noinspection PyTupleAssignmentBalance
         app_name, deployment_name = self.parse_app_deployment_name(args.name)
         if not deployment_name:
             raise InputErrorException('NoDeployment')
         if not args.addon:
             raise InputErrorException('NoAddonGiven')
+
         options = None
         if args.options:
             options = parse_additional_addon_options(args.options)
+
         try:
             self.api.create_addon(app_name, deployment_name, args.addon, options)
         except ConflictDuplicateError:
             raise InputErrorException('DuplicateAddon')
+        except BadRequestError as e:
+            if 'This is not a valid addon name' in str(e):
+                raise InputErrorException('InvalidAddon')
+            raise
+        except ForbiddenError:
+            raise InputErrorException('ForbiddenAddon')
+
         return True
 
     def showAddon(self, args):
         """
             Shows the details of an addon.
         """
-        #noinspection PyTupleAssignmentBalance
         app_name, deployment_name = self.parse_app_deployment_name(args.name)
         if not deployment_name:
             raise InputErrorException('NoDeployment')
@@ -826,7 +834,6 @@ class AppController():
                 return True
 
     def updateAddon(self, args):
-        #noinspection PyTupleAssignmentBalance
         app_name, deployment_name = self.parse_app_deployment_name(args.name)
         if not deployment_name:
             raise InputErrorException('NoDeployment')
@@ -845,7 +852,6 @@ class AppController():
         """
             Removes an addon form a deployment.
         """
-        #noinspection PyTupleAssignmentBalance
         app_name, deployment_name = self.parse_app_deployment_name(args.name)
         if not deployment_name:
             raise InputErrorException('NoDeployment')
@@ -875,8 +881,7 @@ class AppController():
         """
             Add a user specified by the e-mail address to an application.
         """
-        #noinspection PyTupleAssignmentBalance
-        app_name, deployment_name = self.parse_app_deployment_name(args.name)  # @UnusedVariable
+        app_name, deployment_name = self.parse_app_deployment_name(args.name)
         try:
             if deployment_name:
                 self.api.create_deployment_user(app_name, deployment_name, args.email, args.role)
@@ -892,8 +897,7 @@ class AppController():
         """
             Remove a user specified by the user name or email address from an application.
         """
-        #noinspection PyTupleAssignmentBalance
-        app_name, deployment_name = self.parse_app_deployment_name(args.name)  # @UnusedVariable
+        app_name, deployment_name = self.parse_app_deployment_name(args.name)
         if '@' in args.username:
             if deployment_name:
                 users = self.api.read_deployment_users(app_name, deployment_name)
@@ -922,13 +926,11 @@ class AppController():
         """
         Show the log.
         """
-        #noinspection PyTupleAssignmentBalance
         app_name, deployment_name = self.parse_app_deployment_name(args.name)
         if not deployment_name:
             raise InputErrorException('NoDeployment')
         last_time = None
         while True:
-            #noinspection PyUnusedLocal
             logEntries = []
             try:
                 logEntries = self.api.read_log(
@@ -962,33 +964,43 @@ class AppController():
                 print_log_entries(logEntries, args.type)
             time.sleep(2)
 
-    def push(self, args):
-        """
-            Push is actually only a shortcut for bzr and git push commands
-            that automatically takes care of using the correct repository url.
+    def log_from_now(self, app_name, deployment_name, type_):
+        last_time = datetime.now() - timedelta(seconds=10)
+        while True:
+            logEntries = []
+            try:
+                logEntries = self.api.read_log(
+                    app_name,
+                    deployment_name,
+                    type_,
+                    last_time=last_time)
+            except GoneError:
+                raise InputErrorException('WrongApplication')
 
-            It queries the deployment details and uses whatever is in branch.
+            if len(logEntries) > 0:
+                last_time = datetime.fromtimestamp(float(logEntries[-1]["time"]))
+                print_log_entries(logEntries, type_)
 
-            If no deployment exists we automatically create one.
-        """
-        if not check_installed_rcs('bzr') and not check_installed_rcs('git'):
-            raise InputErrorException('NeitherBazaarNorGitFound')
+            time.sleep(1)
 
-        #noinspection PyTupleAssignmentBalance
-        app_name, deployment_name = self.parse_app_deployment_name(args.name)
+    def _get_or_create_deployment(self, app_name, deployment_name, clear_cache):
         try:
             if deployment_name == '':
                 push_deployment_name = 'default'
             else:
                 push_deployment_name = deployment_name
-            #noinspection PyUnusedLocal
+
             deployment = self.api.read_deployment(
                 app_name,
                 push_deployment_name)
+
+            if clear_cache:
+                self._clear_cache(app_name, deployment_name, deployment)
         except GoneError:
             push_deployment_name = ''
             if deployment_name != '':
                 push_deployment_name = deployment_name
+
             try:
                 deployment = self.api.create_deployment(
                     app_name,
@@ -998,25 +1010,31 @@ class AppController():
             except ForbiddenError:
                 raise InputErrorException('NotAllowed')
 
+        return deployment, push_deployment_name
+
+    def _push_cmd(self, deployment, push_deployment_name, source):
         cmd = None
         if deployment['branch'].startswith('bzr+ssh'):
             rcs = check_installed_rcs('bzr')
             if not rcs:
                 raise InputErrorException('BazaarRequiredToPush')
-            if args.source:
-                cmd = [rcs, 'push', deployment['branch'], '-d', args.source]
+
+            if source:
+                cmd = [rcs, 'push', deployment['branch'], '-d', source]
             else:
                 cmd = [rcs, 'push', deployment['branch']]
         elif deployment['branch'].startswith('ssh'):
             rcs = check_installed_rcs('git')
             if not rcs:
                 raise InputErrorException('GitRequiredToPush')
+
             if push_deployment_name == 'default':
                 git_branch = 'master'
             else:
                 git_branch = push_deployment_name
-            if args.source:
-                git_dir = os.path.join(args.source, '.git')
+
+            if source:
+                git_dir = os.path.join(source, '.git')
                 cmd = [
                     rcs,
                     '--git-dir=' + git_dir,
@@ -1025,10 +1043,60 @@ class AppController():
                     git_branch]
             else:
                 cmd = [rcs, 'push', deployment['branch'], git_branch]
+
+        return cmd
+
+    def push(self, args):
+        """
+            Push is actually only a shortcut for bzr and git push commands
+            that automatically takes care of using the correct repository url.
+
+            It queries the deployment details and uses whatever is in branch.
+
+            If deployment exists will clear the buildpack cache if
+            --clear-cache flag is set.
+
+            If no deployment exists we automatically create one.
+
+
+        """
+        if not check_installed_rcs('bzr') and not check_installed_rcs('git'):
+            raise InputErrorException('NeitherBazaarNorGitFound')
+
+        app_name, deployment_name = self.parse_app_deployment_name(args.name)
+        deployment, push_deployment_name = self._get_or_create_deployment(app_name, deployment_name, args.clear_cache)
+
+        cmd = self._push_cmd(deployment, push_deployment_name, args.source)
+
         try:
             check_call(cmd)
         except CalledProcessError, e:
             print str(e)
+            exit(1)
+
+        if args.ship:
+            print
+            print "Deploying newest version. Press Ctrl+C to open {} in your browser.".format(deployment['name'])
+            self.redeploy(deployment)
+
+            try:
+                self.log_from_now(app_name, deployment_name, 'deploy')
+            except KeyboardInterrupt:
+                print
+                print "Opening in browser..."
+                self._open(self._get_url(deployment))
+
+    def _clear_cache(self, app_name, deployment_name, deployment):
+        subdomain = self._get_url(deployment).split('.', 1)[1]
+        host_name = '{}@{}'.format(app_name, subdomain)
+
+        sshopts = shlex.split(os.environ.get('CCTRL_SSHOPTS', ''))
+        ssh_cmd = ['ssh'] + sshopts + ['--', host_name, 'delete-cache', deployment_name]
+
+        try:
+            check_call(ssh_cmd)
+        except CalledProcessError:
+            raise InputErrorException('ClearCacheFailed')
 
     def parse_app_deployment_name(self, name):
         match = re.match(
