@@ -37,6 +37,8 @@ messages['PermissionDenied'] = r'You are not allowed to push to this repository.
 messages['SecurityQuestionDenied'] = r'Action canceled on user request.'
 messages['WrongAddon'] = r'This addon is unknown for this app_name/deployment_name.'
 messages['DuplicateAddon'] = r'You can not add the same addon option twice.'
+messages['InvalidAddon'] = r'This is not a valid addon name. Check the list of available addons with cctrlapp app_name/deployment_name addon.list'
+messages['ForbiddenAddon'] = 'You are not allowed to perform this action.\nIf you are trying to use a Beta addon, you can request access from the addon page.'
 messages['WrongPubKeyName'] = r'The public key file must be named "id_rsa.pub".'
 messages['NoSuchKeyFile'] = r'No such key file. Please check your input.'
 messages['WrongKeyFormat'] = r'Your id_rsa.pub public key file seems to be in the wrong format.'
@@ -68,6 +70,8 @@ messages['AmbiguousSize'] = r'You can only specify one of --size or --memory'
 messages['InvalidMemory'] = r'Memory size should be an integer between 128 and 1024 MB'
 messages['InvalidSize'] = r'Size should be an integer between 1 and 8'
 messages['NoPreviousVersionFound'] = r'Previous version not found.'
+messages['ClearCacheFailed'] = r'Clear buildpack cache failed.'
+messages['DeploymentFailed'] = r'Deployment failed.'
 
 if sys.platform == 'win32':
     messages['UpdateAvailable'] = r'A newer version is available. Please update.'
